## Figure 1.4.  Tray level & vapor flow during a refinery experiment

x <- refinery$Time
y <- refinery$Tray47

# nejprve zvolime "knots" pro splajny
(knots <- c(0, 33, 67, 67, 67, 98, 130, 162, 193))
rozsah <- range(knots)
# rad B-splajnu
norder <- 4

# odtud plyne velikost baze
nbasis <- length (knots) + norder - 2

# Vytvorime bazi splajnu
bbasis <- create.bspline.basis (rozsah, nbasis, norder, knots)

# kolik mame krivek?
n <- 1

# Vytvorime "Lfd" objekty, dve moznosti:
# 1. int2Lfd -- just penalize some derivatives. 
curv.Lfd <- int2Lfd (2)

# 2. vec2Lfd -- a (constant) linear combination of derivatives; for technical
# reasons this also requires the range of the basis. 
# harmLfd <- vec2Lfd (c(1, (2*pi/98)^2, 0), rangeval = rozsah)

#   # definujeme vyhlazovaci parametr lambda
lambda <- 10^(-6)
# 
# # definujeme "fdPar" objekt 
curv.fdPar <- fdPar (bbasis, curv.Lfd, lambda)
# 
W <- data.frame(NULL)

# Jak vybrat lambda?

# cyklus WHILE
lambdas <- 10^(-6)
mean.gcv2 <- mean.gcv1 <- 10^8
# pokud je v dalsim kroku gcv vetsi, skonci se
while (mean.gcv1 >= mean.gcv2){
  mean.gcv1 <- mean.gcv2
  lambdas <- lambdas * 10^(1/2)
  # nastavime lambda
  curv.fdPari <- curv.fdPar
  curv.fdPari$lambda <- lambdas
  
  # hladime
  tempSmoothi <- smooth.basis(x, y, curv.fdPari)
  
  # ulozime prumerne GCV
  mean.gcv2 <- mean(tempSmoothi$gcv)
}

# Vybereme nejlepsi a podle neho nadefinujeme hladkou bazi
lambdabest <- lambdas * 10^(-1/2)
curv.fdPar$lambda <- lambdabest
# hladime
tempSmooth <- smooth.basis(x, y, curv.fdPar)

# A vykreslime vyslednou bazi
xx <- seq(from = x[1], t = x[length(x)], length.out = 50)
fdobjSmooth <- tempSmooth$fd
fdobjSmooth_der1 <- deriv.fd(tempSmooth$fd, 1) # 1. derivace
fdobjSmooth_der2 <- deriv.fd(tempSmooth$fd, 2) # 2. derivace
fdobjSmootheval <- eval.fd(fdobj = fdobjSmooth, evalarg = xx)
meanfdobjSmootheval <- eval.fd(fdobj = mean(fdobjSmooth), evalarg = xx)
fdobjSmootheval_der1 <- eval.fd(fdobj = fdobjSmooth_der1, evalarg = xx)
fdobjSmootheval_der2 <- eval.fd(fdobj = fdobjSmooth_der2, evalarg = xx)
# 2 moznosti pro vykresleni, obe stejne
# par(mfrow=c(2,1))
# matplot(xx, fdobjSmootheval, type = "l")
# plot(fdobjSmooth)

dffda <- data.frame(x = xx, mean = meanfdobjSmootheval, fdacurves = fdobjSmootheval)


x11()
p0 <- ggplot ()
p0 <- p0 + geom_vline(aes(xintercept =  knots[c(2,6:8)]), linetype = "dashed")
p0 <- p0 + geom_vline(aes(xintercept =  knots[3]), linetype = "dashed", size = 1, colour = "red")
p0 <- p0 + geom_point (data = refinery, aes(x = Time, y = Tray47), size = 0.8)
p0 <- p0 + geom_line(data = dffda, aes(x = x, y = mean), size = 1.5)
p0 <- p0 + labs (x = "Time", y = "Tray 47 level") 
p0 <- p0 + theme_bw ()
print (p0)

dev.copy2pdf (file = paste0("rafinery.pdf"), width = 10, height = 7)
